-------------------------------------------------------------------------------- 
-- UPC - EETAC - EEL - CSD - DIGSYS       
--------------------------------------------------------------------------------
-- Counter_BCD_mod60_top prototype targeting a JK_HWD legacy board 
-- Lattice CPLD ispMACH4128V
-- CLK_Generator chip
-- https://digsys.upc.edu/csd/chips/legacy/JK_HWD/Counter_BCD_mod60.html

-- Strategy: Schematic using frequency dividers and T_FF to square signals
---------------------------------------------------
-- Project P8. Lecture L8.2 on designing CLK_generators  
-- https://digsys.upc.edu/csd/P08/L08_2/L8_2.html

-----------------
-- OSC_CLK_in = 1 MHz  from the JK_HWD board oscillator divider
-- To run the Counter_BCD_mod60 with two CLK frequencies:
-- CLK_B_SQ		(20 Hz)  Fast counting
-- CLK_C_SQ 	(1 Hz) 	 Slow counting

-- and CLK_A_SQ		(100 Hz) for running the debouncing filter

--  N1 = 1 MHz / 200 Hz = 5000   (Pulsed_2A = 200 Hz)
--  N2 = 200 Hz / 40 Hz = 5
--  N3 = 40 Hz / 2 Hz 	= 20
--------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
	
ENTITY CLK_Generator IS  
      PORT(	
			OSC_CLK_in	:	IN	STD_LOGIC;
			CD 			:	IN	STD_LOGIC;
			CE			:	IN	STD_LOGIC;
			CLK_B_SQ	:	OUT	STD_LOGIC; -- 20 Hz
			CLK_C_SQ	:	OUT	STD_LOGIC;	-- 1 Hz	
			CLK_A_SQ	:	OUT	STD_LOGIC -- 100 Hz			
		);
END CLK_Generator;

ARCHITECTURE schematic OF CLK_Generator  IS  
-- Components
	COMPONENT Freq_Div_5000	IS                 
		PORT(	
			CD,CLK,CE	: 	IN  std_logic;
			TC5000		:	OUT std_logic
		);
	END COMPONENT;

	COMPONENT Freq_Div_5	IS                 
		PORT(	
			CD,CLK,CE	: 	IN  std_logic;
			TC5			:	OUT std_logic
		);
	END COMPONENT;
	
	COMPONENT Freq_Div_20	IS                 
		PORT(	
			CD,CLK,CE		: 	IN  std_logic;
			TC20			:	OUT std_logic
		);
	END COMPONENT;		
	
	COMPONENT T_FF IS
		Port (	
			CLK 	: IN	STD_LOGIC;
			CD 		: IN	STD_LOGIC;
			T 		: IN	STD_LOGIC;
			Q 		: OUT	STD_LOGIC
		);
	END COMPONENT;					

-- Signals for connecting components together (just the internal wires)
	SIGNAL CLK				:	std_logic;
	SIGNAL Pulsed_40Hz		:	std_logic;
	SIGNAL Pulsed_2Hz		:	std_logic;
	SIGNAL Pulsed_200Hz		:	std_logic;
		
	BEGIN  
-- Instantiation of components

	Chip1	: 	Freq_Div_5000
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK			=> CLK,
			CD 			=> CD,
			CE 			=> CE,
			TC5000		=> Pulsed_200Hz				  
		);
		
	Chip2	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_200Hz,
			Q 		=> CLK_A_SQ				  
		);

	Chip3	: 	Freq_Div_5
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK		=> CLK,
			CD 		=> CD,
			CE 		=> Pulsed_200Hz,
			TC5		=> Pulsed_40Hz					  
		);
		
	Chip4	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_40Hz,
			Q 		=> CLK_B_SQ				  
		);
		
	Chip5	: 	Freq_Div_20
	PORT MAP (	   	
-- from component name 	=> to signal or port name
			CLK			=> CLK,
			CD 			=> CD,
			CE 			=> Pulsed_40Hz,
			TC20		=> Pulsed_2Hz					  
		);	

	Chip6	: T_FF
		PORT MAP (
	-- from component name 	=> to signal or port name	
			CLK		=> CLK,
			CD 		=> CD,
			T		=> Pulsed_2Hz,
			Q 		=> CLK_C_SQ				  
		);		
		
	
-- Connections and logic between components

	-- The internal CLK form the external quartz crystal 
	CLK <= OSC_CLK_in;
	
END schematic ;
